@@ -4,11 +4,11 @@
 #
 # Home page of code is: https://www.smartmontools.org
 #
-# Copyright (C) 2019-21 Christian Franke
+# Copyright (C) 2019-23 Christian Franke
 #
 # SPDX-License-Identifier: GPL-2.0-or-later
 #
-# $Id: cppcheck.sh 5316 2022-02-09 08:02:23Z samm2 $
+# $Id: cppcheck.sh 5489 2023-07-02 17:24:20Z chrfranke $
 #
 
 set -e
@@ -63,7 +63,7 @@ fi
 ver=$("$cppcheck" --version) || exit 1
 ver=${ver##* }
 case $ver in
-  1.8[56]|2.[237]) ;;
+  1.8[56]|2.[237]|2.1[01]) ;;
   *) echo "$myname: cppcheck $ver not tested with this script" ;;
 esac
 
@@ -76,7 +76,11 @@ sup_list="
   #style
   asctime_rCalled:utility.cpp
   asctime_sCalled:utility.cpp
-  ftimeCalled
+  cstyleCast:sg_unaligned.h
+  getgrgidCalled:popen_as_ugid.cpp
+  getgrnamCalled:popen_as_ugid.cpp
+  getpwnamCalled:popen_as_ugid.cpp
+  getpwuidCalled:popen_as_ugid.cpp
   readdirCalled
   strtokCalled
   unusedStructMember
@@ -84,6 +88,17 @@ sup_list="
   unmatchedSuppression
 "
 
+case $ver in
+  2.1[1-9]) sup_list="$sup_list
+  #error
+  ctuOneDefinitionRuleViolation:cissio_freebsd.h
+  ctuOneDefinitionRuleViolation:freebsd_nvme_ioctl.h
+  #information
+  missingInclude
+  missingIncludeSystem
+" ;;
+esac
+
 suppress=
 for s in $sup_list; do
   case $s in
@@ -97,6 +112,7 @@ done
 defs="\
   -U__KERNEL__
   -U__LP64__
+  -U__MINGW64_VERSION_STR
   -U__VERSION__
   -U_NETWARE
   -DBUILD_INFO=\"(...)\"
@@ -104,18 +120,23 @@ defs="\
   -DENOTSUP=1
   -DHAVE_ATTR_PACKED
   -DHAVE_CONFIG_H
+  -DPACKAGE_VERSION=\"7.4\"
   -DSG_IO=1
-  -DSMARTMONTOOLS_SVN_REV=\"r1\"
+  -DSMARTMONTOOLS_BUILD_HOST=\"host\"
   -DSMARTMONTOOLS_ATTRIBUTELOG=\"/file\"
   -DSMARTMONTOOLS_SAVESTATES=\"/file\"
   -DSMARTMONTOOLS_DRIVEDBDIR=\"/dir\"
+  -USMARTMONTOOLS_RELEASE_DATE
+  -USMARTMONTOOLS_RELEASE_TIME
+  -USMARTMONTOOLS_SVN_REV
   -DSOURCE_DATE_EPOCH=1665402854
   -Umakedev
   -Ustricmp"
 
 # Print brief version of command
 cat <<EOF
-cppcheck-$ver \\
+# Cppcheck $ver
+$cppcheck \\
   --enable=$enable \\
   $library \\
   $platform \\
