@@ -38,7 +38,7 @@
 
 #define ARGUSED(x) ((void)(x))
 // Needed by '-V' option (CVS versioning) of smartd/smartctl
-const char *os_darwin_cpp_cvsid="$Id: os_darwin.cpp 5089 2020-10-06 15:31:47Z chrfranke $" \
+const char *os_darwin_cpp_cvsid="$Id: os_darwin.cpp 5209 2021-02-14 18:02:51Z samm2 $" \
 ATACMDS_H_CVSID CONFIG_H_CVSID OS_DARWIN_H_CVSID SCSICMDS_H_CVSID UTILITY_H_CVSID;
 
 // examples for smartctl
@@ -65,7 +65,7 @@ static struct {
   IONVMeSMARTInterface **smartIfNVMe;
 } devices[20];
 
-const char * dev_darwin_cpp_cvsid = "$Id: os_darwin.cpp 5089 2020-10-06 15:31:47Z chrfranke $"
+const char * dev_darwin_cpp_cvsid = "$Id: os_darwin.cpp 5209 2021-02-14 18:02:51Z samm2 $"
   DEV_INTERFACE_H_CVSID;
 
 /////////////////////////////////////////////////////////////////////////////
@@ -85,11 +85,11 @@ public:
 
   virtual ~darwin_smart_device();
 
-  virtual bool is_open() const;
+  virtual bool is_open() const override;
 
-  virtual bool open();
+  virtual bool open() override;
 
-  virtual bool close();
+  virtual bool close() override;
  
 protected:
   /// Return filedesc for derived classes.
@@ -372,7 +372,7 @@ class darwin_ata_device
 {
 public:
   darwin_ata_device(smart_interface * intf, const char * dev_name, const char * req_type);
-  virtual bool ata_pass_through(const ata_cmd_in & in, ata_cmd_out & out);
+  virtual bool ata_pass_through(const ata_cmd_in & in, ata_cmd_out & out) override;
 
 protected:
   // virtual int ata_command_interface(smart_command_set command, int select, char * data);
@@ -398,6 +398,7 @@ bool darwin_ata_device::ata_pass_through(const ata_cmd_in & in, ata_cmd_out & ou
   int fd = get_fd();
   IOATASMARTInterface **ifp = devices[fd].smartIf;
   IOATASMARTInterface *smartIf;
+  io_object_t disk = devices[fd].ioob;
   IOReturn err;
   int timeoutCount = 5;
   int rc = 0;
@@ -429,6 +430,32 @@ bool darwin_ata_device::ata_pass_through(const ata_cmd_in & in, ata_cmd_out & ou
       errno = ENOTSUP;
       err = -1;
       break;
+    case ATA_SET_FEATURES:
+      switch(in.in_regs.features) {
+        case ATA_ENABLE_APM:
+        if (in.in_regs.sector_count) {
+          int l = (int) in.in_regs.sector_count;
+          CFNumberRef cfLevel = CFNumberCreate(kCFAllocatorDefault, kCFNumberSInt32Type, &l);
+          kern_return_t r = IORegistryEntrySetCFProperty(disk, CFSTR("APM Level"), cfLevel);
+          CFRelease(cfLevel);
+          if (r) {
+            switch(r) {
+              case kIOReturnNotPrivileged:
+                return set_err(ENOSYS, "Use superuser to manage APM");
+                break;
+              case kIOReturnUnsupported:
+                return set_err(ENOSYS, "APM not supported");
+                break;
+              default:
+                return set_err(ENOSYS, "APM error: %u", r);
+            }
+          }
+          break;
+        }
+        default:
+          return set_err(ENOSYS, "Unsupported ATA feature");
+        }
+      break;
     case ATA_SMART_CMD:
       switch (in.in_regs.features) {
       case ATA_SMART_READ_VALUES:
@@ -507,22 +534,22 @@ class darwin_smart_interface
 : public /*implements*/ smart_interface
 {
 public:
-  virtual std::string get_os_version_str();
+  virtual std::string get_os_version_str() override;
 
-  virtual std::string get_app_examples(const char * appname);
+  virtual std::string get_app_examples(const char * appname) override;
 
   virtual bool scan_smart_devices(smart_device_list & devlist, const char * type,
-    const char * pattern = 0);
+    const char * pattern = 0) override;
 
 protected:
-  virtual ata_device * get_ata_device(const char * name, const char * type);
+  virtual ata_device * get_ata_device(const char * name, const char * type) override;
   
-  virtual scsi_device * get_scsi_device(const char * name, const char * type);
+  virtual scsi_device * get_scsi_device(const char * name, const char * type) override;
 
   virtual nvme_device * get_nvme_device(const char * name, const char * type,
-    unsigned nsid);
+    unsigned nsid) override;
 
-  virtual smart_device * autodetect_smart_device(const char * name);
+  virtual smart_device * autodetect_smart_device(const char * name) override;
 
 };
 
@@ -537,7 +564,7 @@ public:
   darwin_nvme_device(smart_interface * intf, const char * dev_name,
     const char * req_type, unsigned nsid);
 
-  virtual bool nvme_pass_through(const nvme_cmd_in & in, nvme_cmd_out & out);
+  virtual bool nvme_pass_through(const nvme_cmd_in & in, nvme_cmd_out & out) override;
 };
 
 darwin_nvme_device::darwin_nvme_device(smart_interface * intf, const char * dev_name,
