@@ -15,7 +15,10 @@
 #include <dirent.h>
 #include <stdio.h>
 #include <unistd.h>
+#include <fcntl.h>
 #include <sys/param.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 
 // These are needed to define prototypes for the functions defined below
 #include "config.h"
@@ -24,13 +27,7 @@
 #include "scsicmds.h"
 #include "utility.h"
 
-// This is to include whatever prototypes you define in os_solaris.h
-#include "os_solaris.h"
-
-#define ARGUSED(x) ((void)(x))
-
-const char *os_XXXX_cpp_cvsid = "$Id: os_solaris.cpp 5065 2020-06-20 13:47:36Z chrfranke $"
-  OS_SOLARIS_H_CVSID;
+const char * os_solaris_cpp_cvsid = "$Id: os_solaris.cpp 5393 2022-05-29 05:08:10Z dpgilbert $";
 
 // print examples for smartctl
 void print_smartctl_examples(){
@@ -219,73 +216,11 @@ int deviceclose(int fd){
     return close(fd);
 }
 
-#if defined(WITH_SOLARIS_SPARC_ATA)
-// swap each 2-byte pairs in a sector
-static void swap_sector(void *p)
+// Interface to ATA devices.
+int ata_command_interface(int, smart_command_set, int, char *)
 {
-    int i;
-    char t, *cp = static_cast<char*>(p);
-    for(i = 0; i < 256; i++) {
-        t = cp[0]; cp[0] = cp[1]; cp[1] = t;
-        cp += 2;
-    }
-}
-#endif
-
-// Interface to ATA devices.  See os_linux.c
-int ata_command_interface(int fd, smart_command_set command, int select, char *data){
-#if defined(WITH_SOLARIS_SPARC_ATA)
-    int err;
- 
-    switch (command){
-    case CHECK_POWER_MODE:
-	/* currently not recognized */
-	return -1;
-    case READ_VALUES:
-	return smart_read_data(fd, data);
-    case READ_THRESHOLDS:
-	return smart_read_thresholds(fd, data);
-    case READ_LOG:
-	return smart_read_log(fd, select, 1, data);
-    case IDENTIFY:
-	err = ata_identify(fd, data);
-	if(err) return err;
-	swap_sector(static_cast<void*>(data));
-	return 0;
-    case PIDENTIFY:
-	err = ata_pidentify(fd, data);
-	if(err) return err;
-	swap_sector(static_cast<void*>(data));
-	return 0;
-    case ENABLE:
-	return smart_enable(fd);
-    case DISABLE:
-	return smart_disable(fd);
-    case STATUS:
-	return smart_status(fd);
-    case AUTO_OFFLINE:
-	return smart_auto_offline(fd, select);
-    case AUTOSAVE:
-	return smart_auto_save(fd, select);
-    case IMMEDIATE_OFFLINE:
-	return smart_immediate_offline(fd, select);
-    case STATUS_CHECK:
-	return smart_status_check(fd);
-    default:
-	pout("Unrecognized command %d in ata_command_interface() of os_solaris.cpp\n", command);
-        errno = EINVAL;
-        return -1;
-    }
-#else /* WITH_SOLARIS_SPARC_ATA */
-    ARGUSED(fd); ARGUSED(command); ARGUSED(select); ARGUSED(data);
-
-    /* Above smart_* routines uses undocumented ioctls of "dada"
-     * driver, which is specific to SPARC Solaris.  See
-     * os_solaris_ata.s for further details. */
-
     pout("Device type 'ata' not implemented, try '-d sat' or '-d sat,12' instead.\n");
     errno = ENOSYS;
-#endif
     return -1;
 }
 
@@ -295,7 +230,7 @@ int ata_command_interface(int fd, smart_command_set command, int select, char *d
 #include <sys/scsi/impl/types.h>
 #include <sys/scsi/impl/uscsi.h>
 
-// Interface to SCSI devices.  See os_linux.c
+// Interface to SCSI devices.
 int do_scsi_cmnd_io(int fd, struct scsi_cmnd_io * iop, int report)
 {
   struct uscsi_cmd uscsi;
@@ -305,7 +240,7 @@ int do_scsi_cmnd_io(int fd, struct scsi_cmnd_io * iop, int report)
     const unsigned char * ucp = iop->cmnd;
     const char * np;
 
-    np = scsi_get_opcode_name(ucp[0]);
+    np = scsi_get_opcode_name(ucp);
     pout(" [%s: ", np ? np : "<unknown opcode>");
     for (k = 0; k < (int)iop->cmnd_len; ++k)
       pout("%02x ", ucp[k]);
